@@ -1,4 +1,4 @@
-/*  $Id: ConfigFile.cpp,v 1.14 2005/04/28 23:00:51 terpstra Exp $
+/*  $Id: ConfigFile.cpp,v 1.23 2006/03/01 14:02:51 terpstra Exp $
  *  
  *  ConfigFile.cpp - Knows how to load the config file
  *  
@@ -22,7 +22,6 @@
  *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#define _XOPEN_SOURCE 500
 #define _FILE_OFFSET_BITS 64
 
 #include "ConfigFile.h"
@@ -30,9 +29,14 @@
 
 #include <fstream>
 #include <iostream>
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <time.h>
+#include <stdlib.h>
 
 using namespace std;
 
@@ -43,6 +47,47 @@ lstring::lstring(const string& fallback)
 	s[""] = fallback;
 }
 
+string simplifyPath(string& path)
+{
+	string error = "";
+	
+#ifdef HAVE_REALPATH
+	char clean[PATH_MAX];
+	
+	if (realpath(path.c_str(), clean) != 0)
+	{
+		path = clean;
+		return "";
+	}
+	else
+	{
+		// on linux, realpath files if the last part DNE
+		string dir(path, 0, path.rfind('/'));
+		string tag(path, dir.length(), string::npos);
+		
+		if (realpath(dir.c_str(), clean) != 0)
+		{
+			path = clean + tag;
+			return "";
+		}
+	}
+	
+	// realpath failed, report the problem path
+	error = clean;
+	// fall through with best attempt to clean it up anyways
+#endif
+	
+	// The least we can do is to trim trailing '/'s and simplify '//'s
+	while (path.length() && path[path.length()-1] == '/')
+		path.resize(path.length()-1);
+	
+	string::size_type x = 0;
+	while ((x = path.find("//", x)) != string::npos)
+		path.erase(x, 1);
+	
+	return error;
+}
+
 void lstring::prep_c()
 {
 	c = new map<string, string>();
@@ -333,9 +378,11 @@ bool lstring::is_set() const
 }
 
 Config::Config()
- : list(0), group("lists"), error(), lists(), groups(),
+ : list(0), frontend(0), group(""), error(), lists(), groups(),
+   file(""),
    dbdir(""), 
    xslt("cat -"),
+   delete_message(""),
    pgpv_mime("off"),
    pgpv_inline("off"),
    admin_address(""),
@@ -345,7 +392,6 @@ Config::Config()
    web_cache(true),
    hide_email(false),
    raw_email(true),
-   regroupable(true),
    modified(0)
 {
 }
@@ -456,6 +502,44 @@ int Config::load(const string& file, bool toplevel)
 	
 	if (key != "" && process_command(key, val, dir) != 0) ok = false;
 	
+	if (toplevel)
+	{
+		// do some consistency checks
+		
+		// language field is required
+		Lists::const_iterator i, e;
+		for (i = lists.begin(), e = lists.end(); i != e; ++i)
+		{
+			if (i->second.languages.empty()) 
+			{
+				error << "List '" << i->first << "' has no language!" << endl;
+				return -1;
+			}
+		}
+		
+		// cache directories may not be prefixes
+		Frontends::const_iterator fi, fn, fe;
+		fe = frontends.end();
+		fi = frontends.begin();
+		
+		fn = fi;
+		if (fi != fe) while (1)
+		{
+			++fn;
+			if (fn == fe) break;
+			
+			if (fi->first + "/" == fn->first.substr(0, fi->first.length()+1))
+			{
+				error << "Frontend '" << fi->first << "' is a prefix of '" << fn->first << "', which is forbidden!" << endl;
+				return -1;
+			}
+			
+			fi = fn;
+		}
+		
+		this->file = file;
+	}
+	
 	if (!ok) return -1;
 	return 0;
 }
@@ -515,7 +599,14 @@ int Config::process_command(const string& keys, const string& val, const string&
 			return -1;
 		}
 		
+		if (groups.find(val) != groups.end())
+		{
+			error << "Group id '" << val << "' already exists!" << endl;
+			return -1;
+		}
+		
 		group = val;
+		groups[group]; // make sure it exists
 	}
 	else if (key == "heading")
 	{
@@ -524,6 +615,12 @@ int Config::process_command(const string& keys, const string& val, const string&
 	}
 	else if (key == "list")
 	{
+		if (group == "")
+		{
+			error << "List id '" << val << "' is not a member of any group!" << endl;
+			return -1;
+		}
+		
 		len = 128;
 		if (!isSimple(val) || val.length() == 0)
 		{
@@ -542,13 +639,12 @@ int Config::process_command(const string& keys, const string& val, const string&
 			list = &lists[val];
 			list->mbox = val;
 			list->group = group;
-			list->language = "en";
 			list->offline = false;
 		}
 		else
 		{
-			// re-enter list scope
-			list = &lists[val];
+			error << "List id '" << val << "' already exists!" << endl;
+			return -1;
 		}
 	}
 	else if (key == "title")
@@ -609,7 +705,7 @@ int Config::process_command(const string& keys, const string& val, const string&
 			return -1;
 		}
 		
-		list->language = lval;
+		list->languages.insert(lval);
 	}
 	else if (key == "offline")
 	{
@@ -644,6 +740,116 @@ int Config::process_command(const string& keys, const string& val, const string&
 		
 		list->description.translate(lc, val);
 	}
+	else if (key == "frontend")
+	{
+		len = 10240; // Long paths are ... ok
+		
+		if (lc != "")
+		{
+			error << "frontend path '" <<  val << "' cannot be localized" << endl;
+			return -1;
+		}
+		
+		if (val.length() == 0 || val[0] != '/')
+		{
+			error << "frontend path is not absolute '" << val << "' (must start with a '/')!" << endl;
+			return -1;
+		}
+		
+		// Cleanup the path, but ignore any problems with the path
+		string sval(val);
+		simplifyPath(sval);
+		
+		if (frontends.find(sval) == frontends.end())
+		{
+			frontend = &frontends[sval];
+		}
+		else
+		{
+			error << "Frontend '" << val << "' already exists!" << endl;
+			return -1;
+		}
+	}
+	else if (key == "allow_list")
+	{
+		if (frontend == 0)
+		{
+			error << "No frontend defined for allow_list = '" << val << "'" << endl;
+			return -1;
+		}
+		
+		if (lists.find(val) == lists.end())
+		{
+			error << "List '" << val << "' does not exist for allow_list" << endl;
+			return -1;
+		}
+		
+		Frontend::Entry e;
+		e.perm = Frontend::ALLOW;
+		e.what = Frontend::LIST;
+		e.key = val;
+		frontend->entries.push_back(e);
+	}
+	else if (key == "deny_list")
+	{
+		if (frontend == 0)
+		{
+			error << "No frontend defined for deny_list = '" << val << "'" << endl;
+			return -1;
+		}
+		
+		if (lists.find(val) == lists.end())
+		{
+			error << "List '" << val << "' does not exist for deny_list" << endl;
+			return -1;
+		}
+		
+		Frontend::Entry e;
+		e.perm = Frontend::DENY;
+		e.what = Frontend::LIST;
+		e.key = val;
+		frontend->entries.push_back(e);
+	}
+	else if (key == "allow_group")
+	{
+		if (frontend == 0)
+		{
+			error << "No frontend defined for allow_group = '" << val << "'" << endl;
+			return -1;
+		}
+		
+		if (groups.find(val) == groups.end())
+		{
+			error << "Group '" << val << "' does not exist for allow_group" << endl;
+			return -1;
+		}
+		
+		Frontend::Entry e;
+		e.perm = Frontend::ALLOW;
+		e.what = Frontend::GROUP;
+		e.key = val;
+		frontend->entries.push_back(e);
+	}
+	else if (key == "deny_group")
+	{
+		if (frontend == 0)
+		{
+			error << "No frontend defined for deny_group = '" << val << "'" << endl;
+			return -1;
+		}
+		
+		if (groups.find(val) == groups.end())
+		{
+			error << "Group '" << val << "' does not exist for deny_group" << endl;
+			return -1;
+		}
+		
+		Frontend::Entry e;
+		e.perm = Frontend::DENY;
+		e.what = Frontend::GROUP;
+		e.key = val;
+		frontend->entries.push_back(e);
+	}
 	else if (key == "dbdir")
 	{
 		if (lc != "")
@@ -679,6 +885,15 @@ int Config::process_command(const string& keys, const string& val, const string&
 		}
 		xslt = val;
 	}
+	else if (key == "delete_message")
+	{
+		if (lc != "")
+		{
+			error << "delete_message command cannot be localized" << endl;
+			return -1;
+		}
+		delete_message = val;
+	}
 	else if (key == "pgp_verify_mime")
 	{
 		if (lc != "")
@@ -731,23 +946,6 @@ int Config::process_command(const string& keys, const string& val, const string&
 			return -1;
 		}
 	}
-	else if (key == "regroupable")
-	{
-		if (lc != "")
-		{
-			error << "regroupable cannot be localized" << endl;
-			return -1;
-		}
-		if (val == "off" || val == "false")
-			regroupable = false;
-		else if (val == "on" || val == "true")
-			regroupable = true;
-		else
-		{
-			error << "regroupable must be set to on/off or true/false!" << endl;
-			return -1;
-		}
-	}
 	else if (key == "raw_email")
 	{
 		if (lc != "")
@@ -803,8 +1001,11 @@ ostream& operator << (ostream& o, const List::SerializeMagic& lm)
 	
 	o << "<list>"
 	  << "<id>" << m.mbox << "</id>"
-	  << "<group>" << m.group << "</group>"
-	  << "<language>" << m.language << "</language>";
+	  << "<group>" << m.group << "</group>";
+	
+	set<string>::const_iterator i, e;
+	for (i = m.languages.begin(), e = m.languages.end(); i != e; ++i)
+		o << "<language>" << *i << "</language>";
 	
 	if (m.offline)
 		o << "<offline/>";
@@ -846,10 +1047,10 @@ ostream& operator << (ostream& o, const Config::SerializeMagic& cm)
 	o << "<server>"
 	  << "<version>" << VERSION << "</version>"
 	  << "<eoa-year>" << year << "</eoa-year>"
-	  << "<doc-url>" << c.docUrl << "</doc-url>"
-	  << "<cgi-url>" << c.cgiUrl << "</cgi-url>"
+	  << "<doc-url>" << xmlEscape << c.docUrl << "</doc-url>"
+	  << "<cgi-url>" << xmlEscape << c.cgiUrl << "</cgi-url>"
 	  << "<command>" << c.command << "</command>"
-	  << "<options>" << c.options << "</options>";
+	  << "<options>" << xmlEscape << c.options << "</options>";
 	
 	if (c.raw_email) o << "<raw-email/>";
 	
@@ -868,3 +1069,39 @@ ostream& operator << (ostream& o, const Config::SerializeMagic& cm)
 	
 	return o;
 }
+
+void Config::set_permissions(const Frontend& f)
+{
+	vector<Frontend::Entry>::const_iterator ei,
+		es = f.entries.begin(),
+		ee = f.entries.end();
+	
+	Lists::iterator li,
+		ls = lists.begin(),
+		le = lists.end();
+	
+	// Empty list or first entry deny means default to allow
+	bool def = (es == ee) || (es->perm == Frontend::DENY);
+	for (li = ls; li != le; ++li) li->second.allowed = def;
+	
+	// Walk each entry toggling permissions as we go
+	for (ei = es; ei != ee; ++ei)
+	{
+		bool allowed = (ei->perm == Frontend::ALLOW);
+		
+		if (ei->what == Frontend::LIST)
+		{
+			lists[ei->key].allowed = allowed;
+		}
+		else
+		{ // group
+			Members::const_iterator mi,
+				ms = groups[ei->key].members.begin(),
+				me = groups[ei->key].members.end();
+			for (mi = ms; mi != me; ++mi)
+			{
+				lists[*mi].allowed = allowed;
+			}
+		}
+	}
+}
