@@ -1,4 +1,4 @@
-/*  $Id: main.cpp,v 1.43 2005/04/28 22:57:35 terpstra Exp $
+/*  $Id: main.cpp,v 1.49 2006/03/01 14:11:05 terpstra Exp $
  *  
  *  main.cpp - Read the fed data into our database
  *  
@@ -22,7 +22,6 @@
  *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#define _XOPEN_SOURCE 500
 #define _FILE_OFFSET_BITS 64
 
 #include <mimelib/headers.h>
@@ -35,6 +34,7 @@
 #include <esort.h>
 
 #include <iostream>
+#include <set>
 #include <cstdio>
 #include <ctime>
 #include <cstdlib>
@@ -44,10 +44,13 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <dirent.h>
+#include <errno.h>
 
 #include <zlib.h>
 
 #include "Index.h"
+#include <XmlEscape.h>
+#include <Keys.h>
 
 #include "config.h"
 #ifdef HAVE_SYSEXITS_H
@@ -67,6 +70,7 @@ time_t get_date (const char *p, const time_t *now);
 using namespace std;
 
 auto_ptr<ESort::Writer> db;
+set<MessageId> blacklist;
 
 string         append;
 List*          list = 0;
@@ -77,7 +81,8 @@ time_t start;
 long messagecount = 0;
 off_t processedbytes = 0;
 
-long batch = 0;
+const long batch = 10;
+bool ismbox = true;
 bool verbose = false;
 bool dropdup = false;
 bool synced = true;
@@ -88,22 +93,21 @@ void help(const char* name)
 {
 	cerr << "Lurker-index (v" << VERSION << ") imports messages into the archive.\n";
 	cerr << "\n";
-	cerr << "Usage: " << name << " -c <config-file> -l <list> (-m | -b <count>)\n";
-	cerr << "                                     [-i <mbox/maildir> -v -d -n -u -f]\n";
+	cerr << "Usage: " << name << " -l <list> [-c <config-file>]\n";
+	cerr << "                                     [-m -i <mbox/maildir> -v -d -n -u -f]\n";
 	cerr << "\n";
-	cerr << "\t-c <config-file> Use this config file for lurker settings\n";
 	cerr << "\t-l <list>        Import messages to the named list\n";
-	cerr << "\t-b <count>       Import a batch of messages; flush every count messages\n";
+	cerr << "\t-c <config-file> Use this config file for lurker settings\n";
+	cerr << "\t-m               Input is a single message (not a mailbox)\n";
 	cerr << "\t-i <mbox/mdir>   Read input from mbox or maildir instead of std input\n";
-	cerr << "\t-m               Import a single message\n";
+	cerr << "\t-p <history>     Prevent deleted messages in DB history from importing\n";
 	cerr << "\t-v               Verbose operation\n";
 	cerr << "\t-d               Drop duplicates per list\n";
 	cerr << "\t-n               Don't compress messages\n";
 	cerr << "\t-u               Trust the user Date header more than arrival time\n";
 	cerr << "\t-f               Fast import (but vulnerable to power-failure)\n";
 	cerr << "\n";
-	cerr << "Index messages from standard input and store them in the lurker database.\n";
-	cerr << "Either import a single message, or import a batch of messages in mbox format.\n";
+	cerr << "Index messages and store them in the lurker database.\n";
 	cerr << "\n";
 }
 
@@ -353,7 +357,8 @@ int index(DwString& msg, time_t arrival)
 		append.append(msg.c_str(), msg.length());
 	}
 	
-	Index i(message, db.get(), *list, start, msg.length() + strlen(prefix));
+	Index i(message, db.get(), *list, blacklist, 
+	        start, msg.length() + strlen(prefix));
 	
 	bool exist;
 	if (i.index(userdate, arrival, import, dropdup, exist) != 0)
@@ -440,7 +445,7 @@ string::size_type find_message_boundary(const string& str, string::size_type o)
 		// well, we gave it the benefit of the doubt, but ...
 		// this just don't look like a message break
 		
-		cerr << "warning: treating '" << header << "' as part of a message because it does not have an @ in the username or a valid date." << endl;
+		cerr << "warning: treating '" << xmlEscape << header << "' (string is xml escaped) as part of a message because it does not have an @ in the username or a valid date." << endl;
 		// implicitly continue the loop
 	}
 	
@@ -468,10 +473,9 @@ int process_mail(FILE* file, time_t arrival)
 		
 		string::size_type eos;
 		/* Look for message boundaries only if:
-		 *  We are importing a batch of messsages
-		 *  We are NOT importing a file from a maildir
+		 *  We are importing from an mbox (not single, not maildir)
 		 */
-		while (batch != -1 && arrival == 0 &&
+		while (ismbox && 
 		       (eos = find_message_boundary(buf, was)) != string::npos)
 		{
 			DwString msg(buf.c_str(), eos);
@@ -490,10 +494,9 @@ int process_mail(FILE* file, time_t arrival)
 		if (got == 0)
 		{
 			/* Don't index the empty string!
-			 * If single maildir file or raw delivery or message.
+			 * If not an mbox, or has leading mbox text
 			 */
-			if (buf.length() && 
-			    (batch == -1 || arrival != 0 || buf[0] == 'F'))
+			if (buf.length() && (!ismbox || buf[0] == 'F'))
 			{
 				DwString msg(buf.c_str(), buf.length());
 				buf = "";
@@ -543,13 +546,15 @@ int main(int argc, char** argv)
 {
 	int c;
 	
-	const char* config  = 0;
+	const char* config  = DEFAULT_CONFIG_FILE;
 	const char* listn   = 0;
 	const char* input   = 0;
+	const char* histdb  = 0;
+	bool single = false;
 	
 	srandom(time(0));
 	
-	while ((c = getopt(argc, (char*const*)argv, "c:l:b:i:mvndfu?")) != -1)
+	while ((c = getopt(argc, (char*const*)argv, "c:l:i:p:mvndfu?")) != -1)
 	{
 		switch ((char)c)
 		{
@@ -559,14 +564,14 @@ int main(int argc, char** argv)
 		case 'l':
 			listn = optarg;
 			break;
-		case 'b':
-			batch = atol(optarg);
-			break;
 		case 'i':
 			input = optarg;
 			break;
+		case 'p':
+			histdb = optarg;
+			break;
 		case 'm':
-			batch = -1;
+			single = true;
 			break;
 		case 'v':
 			verbose = true;
@@ -589,9 +594,21 @@ int main(int argc, char** argv)
 		}
 	}
 	
-	if (!config || !listn || batch == 0 || optind < argc)
+	while (optind < argc)
 	{
-		help(argv[0]);
+		if (!argv[optind][0])
+		{	// ignore empty arguments
+			optind++;
+			continue;
+		}
+		
+		cerr << "Unexpected argument: '" << argv[optind] << "'\n";
+		return LEX_USAGE;
+	}
+	
+	if (!listn)
+	{
+		cerr << "No list was specified (the -l option is manditory)\n";
 		return LEX_USAGE;
 	}
 	
@@ -616,15 +633,16 @@ int main(int argc, char** argv)
 		return LEX_DATAERR;
 	}
 	
+	string dbname = cfg.dbdir + "/db";
 	ESort::Parameters params(synced);
 	// work around g++ 2.95 borkage
 	auto_ptr<ESort::Writer> dbt(
-		ESort::Writer::opendb(cfg.dbdir + "/db", params));
+		ESort::Writer::opendb(dbname, params));
 	db = dbt;
 	
 	if (!db.get())
 	{
-		perror("opening database");
+		perror(dbname.c_str());
 		return LEX_IOERR;
 	}
 	
@@ -637,6 +655,41 @@ int main(int argc, char** argv)
 	}
 	length = lseek(mbox, 0, SEEK_END);
 	
+	/* load the blacklist from the old database */
+	if (histdb)
+	{
+		auto_ptr<ESort::Reader> dbh(
+			ESort::Reader::opendb(histdb, params));
+		if (!dbh.get())
+		{
+			perror(histdb);
+			return LEX_IOERR;
+		}
+		
+		string prefix(LU_KEYWORD + string(LU_KEYWORD_DELETED) + '\0');
+		auto_ptr<ESort::Walker> walk(
+			dbh->seek(prefix, "", ESort::Forward));
+		
+		while (walk->advance() != -1)
+		{
+			if (walk->key.length() != 
+			    prefix.length() + MessageId::raw_len)
+			{
+				cerr << histdb << " contains corrupt deleted entries" << endl;
+				return LEX_IOERR;
+			}
+			
+			MessageId id(walk->key.c_str() + prefix.length(), 
+			             MessageId::raw_len);
+			blacklist.insert(id);
+		}
+		if (errno != 0)
+		{
+			perror("walking history database");
+			return LEX_IOERR;
+		}
+	}
+	
 	/** Begin processing input.
 	 */
 	
@@ -646,6 +699,7 @@ int main(int argc, char** argv)
 		DIR* d = opendir((string(input) + "/new").c_str());
 		if (d)
 		{
+			ismbox = false; /* maildirs don't have mboxes */
 			if (maildir(d, 
 				string(input) + "/new",
 				string(input) + "/cur") != 0)
@@ -654,6 +708,7 @@ int main(int argc, char** argv)
 		}
 		else
 		{
+			ismbox = !single;
 			FILE* f = fopen(input, "rb");
 			if (!f)
 			{
@@ -668,6 +723,7 @@ int main(int argc, char** argv)
 	}
 	else
 	{
+		ismbox = !single;
 		if (process_mail(stdin, 0) != 0) return LEX_IOERR;
 	}
 	
