@@ -1,4 +1,4 @@
-/*  $Id: attach.cpp,v 1.11 2005/05/01 15:03:59 terpstra Exp $
+/*  $Id: attach.cpp,v 1.15 2006/03/01 14:55:44 terpstra Exp $
  *  
  *  attach.cpp - Handle a attach/ command
  *  
@@ -22,7 +22,6 @@
  *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#define _XOPEN_SOURCE 500
 #define _FILE_OFFSET_BITS 64
 
 #include <mimelib/message.h>
@@ -40,28 +39,6 @@
 
 using std::cout;
 
-int attach_format_error(const string& param)
-{
-	cout << "Status: 200 OK\r\n";
-	cout <<	"Content-Type: text/html\r\n\r\n";
-	cout << error(_("Bad request"), param,
-		_("The given parameter was not of the correct format. "
-		  "An attach request must be formatted like: "
-		  "attach/id@YYYYMMDD.HHMMSS.hashcode.xml where id "
-		  "is the number of the attachment."));
-	return 1;
-}
-
-int attach_no_permission(const string& param)
-{
-	cout << "Status: 200 OK\r\n";
-	cout <<	"Content-Type: text/html\r\n\r\n";
-	cout << error(_("Permission Denied"), param,
-		_("Access to mail attachments has been disabled. "
-		  "Contact the site administrator if this is a problem."));
-	return 1;
-}
-
 DwEntity& attach_find(DwEntity& e, long& x)
 {
 	// We are the requested entity.
@@ -112,35 +89,39 @@ int handle_attach(const Config& cfg, ESort::Reader* db, const string& param)
 	long n = atol(param.c_str());
 	
 	if (!cfg.raw_email)
-		return attach_no_permission(param);
+		error(_("Permission Denied"), param,
+		      _("Access to mail attachments has been disabled. "
+		        "Contact the site administrator if this is a problem."));
 	
 	if (o == string::npos || n <= 0 || 
 	    !MessageId::is_full(param.c_str()+o+1))
-		return attach_format_error(param);
+		error(_("Bad request"), param,
+		      _("The given parameter was not of the correct format. "
+		        "An attach request must be formatted like: "
+		        "attach/id@YYYYMMDD.HHMMSS.hashcode.xml where id "
+		        "is the number of the attachment."));
 	
 	MessageId id(param.c_str()+o+1);
 	string ok;
 	
 	Summary source(id);
-	if ((ok = source.load(db, cfg)) != "")
+	// Identical error if missing or not allowed
+	if ((ok = source.load(db, cfg)) != "" || !source.allowed())
 	{
-		cout << "Status: 200 OK\r\n";
-		cout <<	"Content-Type: text/html\r\n\r\n";
-		cout << error(_("Database attach source pull failure"), ok,
-			_("The specified message does not exist."));
-		return 1;
+		if (ok == "") ok = "not in a mailbox"; // fake
+		error(_("Database attach source pull failure"), ok,
+		      _("The specified message does not exist."));
 	}
 	
+	if (source.deleted())
+		error(_("Database attach source pull failure"), "not found",
+		      _("The specified message has been deleted."));
+	
 	DwMessage message;
 	if ((ok = source.message(cfg.dbdir, message)) != "")
-	{
-		cout << "Status: 200 OK\r\n";
-		cout <<	"Content-Type: text/html\r\n\r\n";
-		cout << error(_("MBox read failure"), ok,
-			_("Unable to open message in the mailbox. "
-			  "Perhaps it has been deleted or moved?"));
-		return 1;
-	}
+		error(_("MBox read failure"), ok,
+		      _("Unable to open message in the mailbox. "
+		        "Perhaps it has been deleted or moved?"));
 	
 	DwEntity& e = attach_find(message, n);
 	
@@ -151,10 +132,8 @@ int handle_attach(const Config& cfg, ESort::Reader* db, const string& param)
 	
 	// if (e.hasHeaders() &&
 	if (e.Headers().HasContentType())
-	{
 		cout << unfold_header(
 			e.Headers().ContentType().AsString().c_str());
-	}
 	else	cout <<	"text/plain";
 	cout << "\r\n";
 	
