@@ -1,4 +1,4 @@
-/*  $Id: message.cpp,v 1.39 2004/08/27 15:04:05 terpstra Exp $
+/*  $Id: message.cpp,v 1.47 2006/03/01 14:55:45 terpstra Exp $
  *  
  *  message.cpp - Handle a message/ command
  *  
@@ -22,7 +22,6 @@
  *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#define _XOPEN_SOURCE 500
 #define _FILE_OFFSET_BITS 64
 
 #include <mimelib/headers.h>
@@ -180,7 +179,7 @@ void my_service_quote(
 	my_service_pic(o, s, e-s, cfg);
 }
 
-void find_and_replace(string& target, const string& token, const string& value)
+static void find_and_replace(string& target, const string& token, const string& value)
 {
 	string::size_type x = 0;
 	while ((x = target.find(token, x)) != string::npos)
@@ -675,15 +674,10 @@ int handle_message(const Config& cfg, ESort::Reader* db, const string& param)
 	
 	if (!MessageId::is_full(req.options.c_str()) ||
 	    req.options.length() != MessageId::full_len)
-	{
-		cout << "Status: 200 OK\r\n";
-		cout <<	"Content-Type: text/html\r\n\r\n";
-		cout << error(_("Bad request"), param,
-			_("The given parameter was not of the correct format. "
-			  "A message request must be formatted like: "
-			  "message/YYYYMMDD.HHMMSS.hashcode.lc.xml"));
-		return 1;
-	}
+		error(_("Bad request"), param,
+		      _("The given parameter was not of the correct format. "
+		        "A message request must be formatted like: "
+		        "message/YYYYMMDD.HHMMSS.hashcode.lc.xml"));
 	
 	MessageId id(req.options.c_str());
 	
@@ -693,66 +687,42 @@ int handle_message(const Config& cfg, ESort::Reader* db, const string& param)
 	string ok;
 	
 	Summary source(id);
-	if ((ok = source.load(db, cfg)) != "")
+	// Identical error if missing or forbidden (security)
+	if ((ok = source.load(db, cfg)) != "" || !source.allowed())
 	{
-		cout << "Status: 200 OK\r\n";
-		cout <<	"Content-Type: text/html\r\n\r\n";
-		cout << error(_("Database message source pull failure"), ok,
-			_("The specified message does not exist."));
-		return 1;
+		if (ok == "") ok = "not in a mailbox"; // fake
+		error(_("Database message source pull failure"), ok,
+		      _("The specified message does not exist."));
 	}
 	
-	// No keywords means implicitly search for everything
-	vector<Summary> dead;
-	Search e(cfg, db, Forward, id);
-	if (!e.pull(1, dead) || dead.empty() || dead[0].id() != id)
-	{
-		cout << "Status: 200 OK\r\n";
-		cout <<	"Content-Type: text/html\r\n\r\n";
-		cout << error(_("Database message source pull failure"), "not found",
-			_("The specified message has been deleted."));
-		return 1;
-	}
+	if (source.deleted())
+		error(_("Database message source pull failure"), "not found",
+		      _("The specified message has been deleted."));
 	
 	Threading::Key spot;
 	Threading thread;
 	if ((ok = thread.load(db, source, spot)) != "" ||
 	    (ok = thread.draw_snippet(db, spot, cfg)) != "")
-	{
-		cout << "Status: 200 OK\r\n";
-		cout <<	"Content-Type: text/html\r\n\r\n";
-		cout << error(_("Database message tree load failure"), ok,
-			_("Something internal to the database failed. "
-			  "Please contact the lurker user mailing list for "
-			  "further assistence."));
-		return 1;
-	}
+		error(_("Database message tree load failure"), ok,
+		      _("Something internal to the database failed. "
+		        "Please contact the lurker user mailing list for "
+		        "further assistence."));
 	
 	Summary thread_prev;
 	Summary thread_next;
 	
 	if ((ok = thread.findprev(spot, db, cfg, thread_prev)) != "" ||
 	    (ok = thread.findnext(spot, db, cfg, thread_next)) != "")
-	{
-		cout << "Status: 200 OK\r\n";
-		cout <<	"Content-Type: text/html\r\n\r\n";
-		cout << error(_("Thread prev/next load failure"), ok,
-			_("Something internal to the database failed. "
-			  "Please contact the lurker user mailing list for "
-			  "further assistence."));
-		return 1;
-	}
+		error(_("Thread prev/next load failure"), ok,
+		      _("Something internal to the database failed. "
+		        "Please contact the lurker user mailing list for "
+		        "further assistence."));
 	
 	DwMessage message;
 	if ((ok = source.message(cfg.dbdir, message)) != "")
-	{
-		cout << "Status: 200 OK\r\n";
-		cout <<	"Content-Type: text/html\r\n\r\n";
-		cout << error(_("MBox read failure"), ok,
-			_("Unable to open message in the mailbox. "
-			  "Perhaps it has been deleted or moved?"));
-		return 1;
-	}
+		error(_("MBox read failure"), ok,
+		      _("Unable to open message in the mailbox. "
+		        "Perhaps it has been deleted or moved?"));
 	
 	map<string, Summary> followups; // these are all followups NOT in the tree
 	if (message.Headers().HasMessageId())
@@ -787,15 +757,10 @@ int handle_message(const Config& cfg, ESort::Reader* db, const string& param)
 		}
 	}
 	if (ok != "")
-	{
-		cout << "Status: 200 OK\r\n";
-		cout <<	"Content-Type: text/html\r\n\r\n";
-		cout << error(_("Database followups load failure"), ok,
-			_("Something internal to the database failed. "
-			  "Please contact the lurker user mailing list for "
-			  "further assistence."));
-		return 1;
-	}
+		error(_("Database followups load failure"), ok,
+		      _("Something internal to the database failed. "
+		        "Please contact the lurker user mailing list for "
+		        "further assistence."));
 	
 	map<string, Summary> repliesTo; // what messages this one replies to
 	if (message.Headers().HasInReplyTo())
@@ -829,42 +794,34 @@ int handle_message(const Config& cfg, ESort::Reader* db, const string& param)
 		}
 	}
 	if (ok != "")
-	{
-		cout << "Status: 200 OK\r\n";
-		cout <<	"Content-Type: text/html\r\n\r\n";
-		cout << error(_("Database replies load failure"), ok,
-			_("Something internal to the database failed. "
-			  "Please contact the lurker user mailing list for "
-			  "further assistence."));
-		return 1;
-	}
+		error(_("Database replies load failure"), ok,
+		      _("Something internal to the database failed. "
+		        "Please contact the lurker user mailing list for "
+		        "further assistence."));
 	
 	vector<MBox> boxes;
 	set<string>::iterator mbox;
 	for (mbox = source.mboxs().begin(); mbox != source.mboxs().end(); ++mbox)
 	{
 		Config::Lists::const_iterator j = cfg.lists.find(*mbox);
-		if (j == cfg.lists.end()) continue;
+		if (j == cfg.lists.end()) continue; // impossible!
+		if (!j->second.allowed) continue;
 		
 		boxes.push_back(MBox(j->second));
 		if ((ok = boxes.back().load(db, id, cfg)) != "") break;
 	}
 	if (ok != "")
-	{
-		cout << "Status: 200 OK\r\n";
-		cout <<	"Content-Type: text/html\r\n\r\n";
-		cout << error(_("Database list links load failure"), ok,
-			_("Something internal to the database failed. "
-			  "Please contact the lurker user mailing list for "
-			  "further assistence."));
-		return 1;
-	}
+		error(_("Database list links load failure"), ok,
+		      _("Something internal to the database failed. "
+		        "Please contact the lurker user mailing list for "
+		        "further assistence."));
 	
 	Cache cache(cfg, "message", param, req.ext);
 	
 	cache.o << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
-		<< "<?xml-stylesheet type=\"text/xsl\" href=\"../fmt/message.xsl\"?>\n"
+		<< "<?xml-stylesheet type=\"text/xsl\" href=\"../ui/message.xsl\"?>\n"
 		<< "<message xml:lang=\"" << req.language << "\">\n"
+		<< " <mode>" << req.ext << "</mode>\n"
 		<< " " << cfg(req.language) << "\n"
 		<< " " << source << "\n";
 	
@@ -982,6 +939,15 @@ int handle_message(const Config& cfg, ESort::Reader* db, const string& param)
 	
 	cache.o << " </threading>\n";
 	
+	if (message.Headers().HasMessageId())
+	{
+		vector<string> mids = extract_message_ids(
+			message.Headers().MessageId().AsString().c_str());
+		if (mids.size() > 0)
+			cache.o << " <message-id>" << xmlEscape << mids[0] 
+				<< "</message-id>\n";
+	}
+	
 	long aid = 0;
 	// default charset is ISO-8859-1
 	message_build(cache.o, message, "ISO-8859-1", true, aid, cfg);
