@@ -1,4 +1,4 @@
-/*  $Id: parse.h,v 1.2 2003/04/21 18:26:21 terpstra Exp $
+/*  $Id: parse.h,v 1.5 2006/03/01 14:19:04 terpstra Exp $
  *  
  *  parse.h - Deal with CGI ugliness
  *  
@@ -30,7 +30,12 @@
 
 using namespace std;
 
-map<string, string>	getParams();
-int			redirectUrl(const string& url);
+string decipherHalf(const string& str);
+
+map<string, string> getParams();
+map<string, string> getCookies();
+
+string uriEncode(const string& str);
+int redirectUrl(const string& url);
 
 #endif
